@@ -1,11 +1,12 @@
+from collections import deque
 from dataclasses import dataclass
 from enum import Enum, auto
-from typing import Callable, List
+from typing import Callable, Iterable, List
 
 import numpy as np
 import pandas as pd
 
-from .doc_handlers import BaseHandler
+from .handlers.base_handler import BaseHandler, BaseNode, TreeFileHandler
 from .util import match_regex_list, str_comparison
 
 
@@ -22,7 +23,16 @@ class FieldOrientation(Enum):
 
 
 class Extractor:
+    """The Extractor class has the functions to extract data from tables in document.
+    It receives a document handler on initialization, this handler follows an interface, so the extraction is agnostic of the document type.
+    """
+
     def __init__(self, doc_handler: BaseHandler) -> None:
+        """Initialize the extractor with a document handler
+
+        Args:
+            doc_handler (BaseHandler): document handler to extract data from
+        """
         self.doc_handler = doc_handler
 
     def extract_closest_table(
@@ -30,16 +40,18 @@ class Extractor:
         search_headers: List[str],
         validation_funtion: Callable[[List[str]], bool] = lambda x: True,
         minimum_proximity_ratio: float = 0,
+        rename_columns: bool = True,
     ) -> pd.DataFrame:
         """Extract the table in document that has the closest header to search_headers
 
         Args:
-            - search_headers (List[str]): list of itens to search in header.
-            - validation_funtion (Callable[[List[str]], bool], optional): function to validate if the table is valid. This function receives the table header as argument and must return True if the table is valid. Defaults to lambda x: True.
-            - minimum_proximity_ratio (float, optional): minimum proximity ratio to consider there is a match in header. Value must be between 0 and 100. Defaults to 0.
+            search_headers (List[str]): list of itens to search in header.
+            validation_funtion (Callable[[List[str]], bool], optional): function to validate if the table is valid. This function receives the table header as argument and must return True if the table is valid. Defaults to lambda x: True.
+            minimum_proximity_ratio (float, optional): minimum proximity ratio to consider there is a match in header. Value must be between 0 and 100. Defaults to 0.
+            rename_columns (bool, optional): if true, the returned dataframe will have its columns renamed to match the search_headers. Defaults to True.
 
         Returns:
-            - pd.DataFrame: best match
+            pd.DataFrame: best match
         """
         if minimum_proximity_ratio < 0 or minimum_proximity_ratio > 100:
             raise ValueError("minimum_proximity_ratio must be between 0 and 100")
@@ -63,9 +75,9 @@ class Extractor:
         if best_ratio < minimum_proximity_ratio:
             return pd.DataFrame()
 
-        best_match = tables[np.argmax(ratios)]
-
-        df = self.get_columns_fuzzy(best_match, search_headers)
+        df = tables[np.argmax(ratios)]
+        if rename_columns:
+            df = self.get_columns_fuzzy(df, search_headers)
 
         return df
 
@@ -81,14 +93,14 @@ class Extractor:
         The data will be extracted from tables in document
 
         Args:
-            - field (str): search field
-            - orientation (FieldOrientation): orientation to search the content of field
-            - regex (List[str], optional): list of regex to apply to content. To be a valid content there must be at least one match of regex in list. Defaults to [''].
-            - title_regex (List[str], optional): list of regex to apply to title. To be a valid title there must be at least one match of regex in list. Defaults to [''].
-            - return_multiple (bool, optional): if True, will return all matches that has the same proximity ratio. Defaults to False.
+            field (str): search field
+            orientation (FieldOrientation): orientation to search the content of field
+            regex (List[str], optional): list of regex to apply to content. To be a valid content there must be at least one match of regex in list. Defaults to [''].
+            title_regex (List[str], optional): list of regex to apply to title. To be a valid title there must be at least one match of regex in list. Defaults to [''].
+            return_multiple (bool, optional): if True, will return all matches that has the same proximity ratio. Defaults to False.
 
         Returns:
-            - str: best match
+            str: best match
         """
         df = self.doc_handler.dictionary
 
@@ -121,11 +133,11 @@ class Extractor:
         """Calculates a proximity ratio of two headers
 
         Args:
-            - document_headers (List[str]): headers in document
-            - search_headers (List[str]): search headers
+            document_headers (List[str]): headers in document
+            search_headers (List[str]): search headers
 
         Returns:
-            - int: proximity ratio
+            int: proximity ratio
         """
         matches = Extractor.headers_association(document_headers, search_headers)
 
@@ -144,11 +156,11 @@ class Extractor:
         """Determine the best association of two headers
 
         Args:
-            - document_headers (List[str]): headers in document
-            - search_headers (List[str]): search headers
+            document_headers (List[str]): headers in document
+            search_headers (List[str]): search headers
 
         Returns:
-            - List[TableMatch]: list of table headers matches
+            List[TableMatch]: list of table headers matches
         """
         if len(search_headers) > len(document_headers):
             return []
@@ -181,12 +193,12 @@ class Extractor:
         The columns will be renamed to match the closest column name
 
         Args:
-            - df (pd.DataFrame): dataframe to search columns
-            - columns (List[str]): columns to search
-            - threshold (int, optional): minimum score to consider a match. Defaults to 0.
+            df (pd.DataFrame): dataframe to search columns
+            columns (List[str]): columns to search
+            threshold (int, optional): minimum score to consider a match. Defaults to 0.
 
         Returns:
-            - List[str]: columns that match
+            List[str]: columns that match
         """
         association = Extractor.headers_association(df.columns.to_list(), columns)
 
@@ -199,3 +211,49 @@ class Extractor:
         df.rename(columns=rename_dict, inplace=True)
 
         return df
+
+
+@dataclass
+class _SectionPath:
+    nodes: List[BaseNode]
+    score: float = 0
+
+    def add_node(self, node: BaseNode, section_names: List[str]):
+        name = section_names[len(self.nodes) - 1]
+        self.score += str_comparison(name, node.title)
+
+
+class TreeExtractor(Extractor):
+    def __init__(self, doc_handler: TreeFileHandler):
+        super().__init__(doc_handler)
+        self.doc_handler = doc_handler
+
+    def get_closest_section(self, titles: List[str]) -> BaseNode:
+        """Get the closest section given a list of section titles.
+        To reach the closest section all possible paths will be explored, the best path
+        is the one with the highest sum of title comparison scores.
+
+        Args:
+            titles (List[str]): list of section titles
+
+        Returns:
+            BaseNode: closest section
+        """
+
+        initial_path = _SectionPath(nodes=[self.doc_handler.root], score=0)
+        paths = deque([initial_path])
+        valid_paths = []
+
+        while paths:
+            path = paths.popleft()
+            for node in path.nodes:
+                new_path = _SectionPath(nodes=path.nodes[:], score=path.score)
+                new_path.add_node(node, titles)
+
+                if len(new_path.nodes) == len(titles) + 1:
+                    valid_paths.append(new_path)
+                else:
+                    paths.append(new_path)
+
+        best_path = max(valid_paths, key=lambda x: x.score)
+        return best_path
